"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.attachClipboardStubToView = attachClipboardStubToView;
exports.createClipboardItem = createClipboardItem;
exports.detachClipboardStubFromView = detachClipboardStubFromView;
exports.readDataTransferFromClipboard = readDataTransferFromClipboard;
exports.resetClipboardStubOnView = resetClipboardStubOnView;
exports.writeDataTransferToClipboard = writeDataTransferToClipboard;

var _ = require("..");

// Clipboard is not available in jsdom
class ClipboardItemStub {
  constructor(data) {
    this.data = void 0;
    this.data = data;
  }

  get types() {
    return Array.from(Object.keys(this.data));
  }

  async getType(type) {
    const data = await this.data[type];

    if (!data) {
      throw new Error(`${type} is not one of the available MIME types on this item.`);
    }

    return data instanceof Blob ? data : new Blob([data], {
      type
    });
  }

}

const ClipboardStubControl = Symbol('Manage ClipboardSub');

class ClipboardStub extends EventTarget {
  constructor(...args) {
    super(...args);
    this.items = [];
    this[ClipboardStubControl] = void 0;
  }

  async read() {
    return Array.from(this.items);
  }

  async readText() {
    let text = '';

    for (const item of this.items) {
      const type = item.types.includes('text/plain') ? 'text/plain' : item.types.find(t => t.startsWith('text/'));

      if (type) {
        text += await item.getType(type).then(b => (0, _.readBlobText)(b));
      }
    }

    return text;
  }

  async write(data) {
    this.items = data;
  }

  async writeText(text) {
    this.items = [createClipboardItem(text)];
  }

} // MDN lists string|Blob|Promise<Blob|string> as possible types in ClipboardItemData
// lib.dom.d.ts lists only Promise<Blob|string>
// https://developer.mozilla.org/en-US/docs/Web/API/ClipboardItem/ClipboardItem#syntax


function createClipboardItem(...blobs) {
  // use real ClipboardItem if available
  // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
  const constructor = typeof ClipboardItem === 'undefined' ? ClipboardItemStub :
  /* istanbul ignore next */
  ClipboardItem;
  return new constructor(Object.fromEntries(blobs.map(b => [typeof b === 'string' ? 'text/plain' : b.type, Promise.resolve(b)])));
}

function attachClipboardStubToView(window) {
  if (window.navigator.clipboard instanceof ClipboardStub) {
    return window.navigator.clipboard[ClipboardStubControl];
  }

  const realClipboard = Object.getOwnPropertyDescriptor(window.navigator, 'clipboard');
  let stub = new ClipboardStub();
  const control = {
    resetClipboardStub: () => {
      stub = new ClipboardStub();
      stub[ClipboardStubControl] = control;
    },
    detachClipboardStub: () => {
      /* istanbul ignore if */
      if (realClipboard) {
        Object.defineProperty(window.navigator, 'clipboard', realClipboard);
      } else {
        Object.defineProperty(window.navigator, 'clipboard', {
          value: undefined,
          configurable: true
        });
      }
    }
  };
  stub[ClipboardStubControl] = control;
  Object.defineProperty(window.navigator, 'clipboard', {
    get: () => stub,
    configurable: true
  });
  return stub[ClipboardStubControl];
}

function resetClipboardStubOnView(window) {
  if (window.navigator.clipboard instanceof ClipboardStub) {
    window.navigator.clipboard[ClipboardStubControl].resetClipboardStub();
  }
}

function detachClipboardStubFromView(window) {
  if (window.navigator.clipboard instanceof ClipboardStub) {
    window.navigator.clipboard[ClipboardStubControl].detachClipboardStub();
  }
}

async function readDataTransferFromClipboard(document) {
  var _document$defaultView;

  const clipboard = (_document$defaultView = document.defaultView) == null ? void 0 : _document$defaultView.navigator.clipboard;
  const items = clipboard && (await clipboard.read());

  if (!items) {
    throw new Error('The Clipboard API is unavailable.');
  }

  const dt = (0, _.createDataTransfer)();

  for (const item of items) {
    for (const type of item.types) {
      dt.setData(type, await item.getType(type).then(b => (0, _.readBlobText)(b)));
    }
  }

  return dt;
}

async function writeDataTransferToClipboard(document, clipboardData) {
  var _document$defaultView2;

  const clipboard = (_document$defaultView2 = document.defaultView) == null ? void 0 : _document$defaultView2.navigator.clipboard;
  const items = [];

  for (let i = 0; i < clipboardData.items.length; i++) {
    const dtItem = clipboardData.items[i];
    const blob = (0, _.getBlobFromDataTransferItem)(dtItem);
    items.push(createClipboardItem(blob));
  }

  const written = clipboard && (await clipboard.write(items).then(() => true, // Can happen with other implementations that e.g. require permissions

  /* istanbul ignore next */
  () => false));

  if (!written) {
    throw new Error('The Clipboard API is unavailable.');
  }
}
/* istanbul ignore else */
// eslint-disable-next-line @typescript-eslint/no-unnecessary-condition


if (afterEach) {
  afterEach(() => resetClipboardStubOnView(window));
}
/* istanbul ignore else */
// eslint-disable-next-line @typescript-eslint/no-unnecessary-condition


if (afterAll) {
  afterAll(() => detachClipboardStubFromView(window));
}