"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createDataTransfer = createDataTransfer;
exports.getBlobFromDataTransferItem = getBlobFromDataTransferItem;

var _FileList = require("./FileList");

// DataTransfer is not implemented in jsdom.
// DataTransfer with FileList is being created by the browser on certain events.
class DataTransferItemStub {
  constructor(dataOrFile, type) {
    this.kind = void 0;
    this.type = void 0;
    this.file = null;
    this.data = undefined;

    if (typeof dataOrFile === 'string') {
      this.kind = 'string';
      this.type = String(type);
      this.data = dataOrFile;
    } else {
      this.kind = 'file';
      this.type = dataOrFile.type;
      this.file = dataOrFile;
    }
  }

  getAsFile() {
    return this.file;
  }

  getAsString(callback) {
    if (typeof this.data === 'string') {
      callback(this.data);
    }
  }
  /* istanbul ignore next */


  webkitGetAsEntry() {
    throw new Error('not implemented');
  }

}

class DataTransferItemListStub extends Array {
  add(...args) {
    const item = new DataTransferItemStub(args[0], args[1]);
    this.push(item);
    return item;
  }

  clear() {
    this.splice(0, this.length);
  }

  remove(index) {
    this.splice(index, 1);
  }

}

function getTypeMatcher(type, exact) {
  const [group, sub] = type.split('/');
  const isGroup = !sub || sub === '*';
  return item => {
    return exact ? item.type === (isGroup ? group : type) : isGroup ? item.type.startsWith(`${group}/`) : item.type === group;
  };
}

class DataTransferStub {
  constructor() {
    this.dropEffect = 'none';
    this.effectAllowed = 'uninitialized';
    this.items = new DataTransferItemListStub();
    this.files = (0, _FileList.createFileList)([]);
  }

  getData(format) {
    var _this$items$find;

    const match = (_this$items$find = this.items.find(getTypeMatcher(format, true))) != null ? _this$items$find : this.items.find(getTypeMatcher(format, false));
    let text = '';
    match == null ? void 0 : match.getAsString(t => {
      text = t;
    });
    return text;
  }

  setData(format, data) {
    const matchIndex = this.items.findIndex(getTypeMatcher(format, true));
    const item = new DataTransferItemStub(data, format);

    if (matchIndex >= 0) {
      this.items.splice(matchIndex, 1, item);
    } else {
      this.items.push(item);
    }
  }

  clearData(format) {
    if (format) {
      const matchIndex = this.items.findIndex(getTypeMatcher(format, true));

      if (matchIndex >= 0) {
        this.items.remove(matchIndex);
      }
    } else {
      this.items.clear();
    }
  }

  get types() {
    const t = [];

    if (this.files.length) {
      t.push('Files');
    }

    this.items.forEach(i => t.push(i.type));
    Object.freeze(t);
    return t;
  }
  /* istanbul ignore next */


  setDragImage() {}

}

function createDataTransfer(files = []) {
  // Use real DataTransfer if available
  // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
  const dt = typeof DataTransfer === 'undefined' ? new DataTransferStub() :
  /* istanbul ignore next */
  new DataTransfer();
  Object.defineProperty(dt, 'files', {
    get: () => (0, _FileList.createFileList)(files)
  });
  return dt;
}

function getBlobFromDataTransferItem(item) {
  if (item.kind === 'file') {
    return item.getAsFile();
  }

  let data = '';
  item.getAsString(s => {
    data = s;
  });
  return new Blob([data], {
    type: item.type
  });
}